@@ -1,10 +1,8 @@
-/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
- */
-
-#ifndef __SIEVE_TYPES_H
-#define __SIEVE_TYPES_H
+#ifndef SIEVE_TYPES_H
+#define SIEVE_TYPES_H
 
 #include "lib.h"
+#include "smtp-address.h"
 
 #include <stdio.h>
 
@@ -12,6 +10,9 @@
  * Forward declarations
  */
 
+struct smtp_params_mail;
+struct smtp_params_rcpt;
+
 struct sieve_instance;
 struct sieve_callbacks;
 
@@ -63,6 +64,8 @@ struct sieve_environment {
 	const char *home_dir;
 	const char *temp_dir;
 
+	struct event *event_parent;
+
 	enum sieve_flag flags;
 	enum sieve_env_location location;
 	enum sieve_delivery_phase delivery_phase;
@@ -129,11 +132,17 @@ enum sieve_compile_flags {
 
 struct sieve_message_data {
 	struct mail *mail;
-	const char *return_path;
-	const char *orig_envelope_to;
-	const char *final_envelope_to;
+
 	const char *auth_user;
 	const char *id;
+
+	struct {
+		const struct smtp_address *mail_from;
+		const struct smtp_params_mail *mail_params;
+
+		const struct smtp_address *rcpt_to;
+		const struct smtp_params_rcpt *rcpt_params;
+	} envelope;
 };
 
 /*
@@ -148,7 +157,9 @@ enum sieve_execute_flags {
 	/* Do not execute (implicit keep) at the end */
 	SIEVE_EXECUTE_FLAG_DEFER_KEEP = (1<<1),
 	/* There is no envelope */
-	SIEVE_EXECUTE_FLAG_NO_ENVELOPE = (1<<2)
+	SIEVE_EXECUTE_FLAG_NO_ENVELOPE = (1<<2),
+	/* Skip sending responses */
+	SIEVE_EXECUTE_FLAG_SKIP_RESPONSES = (1<<3),
 };
 
 /*
@@ -182,8 +193,8 @@ struct sieve_trace_config {
 struct sieve_script_env {
 	/* Mail-related */
 	struct mail_user *user;
+	const struct message_address *postmaster_address;
 	const char *default_mailbox;
-	const char *postmaster_address;
 	bool mailbox_autocreate;
 	bool mailbox_autosubscribe;
 
@@ -195,15 +206,19 @@ struct sieve_script_env {
 
 	/* Interface for sending mail */
 	void *(*smtp_start)
-		(const struct sieve_script_env *senv, const char *return_path);
+		(const struct sieve_script_env *senv,
+			const struct smtp_address *mail_from);
 	/* Add a new recipient */
 	void (*smtp_add_rcpt)	
 		(const struct sieve_script_env *senv, void *handle,
-			const char *address);
+			const struct smtp_address *rcpt_to);
 	/* Get an output stream where the message can be written to. The recipients
 	   must already be added before calling this. */
 	struct ostream *(*smtp_send)
 		(const struct sieve_script_env *senv, void *handle);
+	/* Abort the SMTP transaction after smtp_send() is already issued */
+	void (*smtp_abort)
+		(const struct sieve_script_env *senv, void *handle);
 	/* Returns 1 on success, 0 on permanent failure, -1 on temporary failure. */
 	int (*smtp_finish)
 		(const struct sieve_script_env *senv, void *handle,
@@ -220,7 +235,13 @@ struct sieve_script_env {
 
 	/* Interface for rejecting mail */
 	int (*reject_mail)(const struct sieve_script_env *senv,
-		const char *recipient, const char *reason);
+		const struct smtp_address *recipient, const char *reason);
+
+	/* Interface for amending result messages */
+	const char *
+	(*result_amend_log_message)(const struct sieve_script_env *senv,
+				    enum log_type log_type,
+				    const char *message);
 
 	/* Execution status record */
 	struct sieve_exec_status *exec_status;
@@ -240,11 +261,12 @@ struct sieve_script_env {
 struct sieve_exec_status {
 	struct mail_storage *last_storage;
 
-	unsigned int message_saved:1;
-	unsigned int message_forwarded:1;
-	unsigned int tried_default_save:1;
-	unsigned int keep_original:1;
-	unsigned int store_failed:1;
+	bool message_saved:1;
+	bool message_forwarded:1;
+	bool tried_default_save:1;
+	bool keep_original:1;
+	bool store_failed:1;
+	bool significant_action_executed:1;
 };
 
 /*
@@ -259,4 +281,4 @@ enum sieve_execution_exitcode {
 	SIEVE_EXEC_KEEP_FAILED  = -3
 };
 
-#endif /* __SIEVE_TYPES_H */
+#endif
